#include "windows.h"
#include "stdio.h"
#include "resource.h"
#include "Commctrl.h"
#include <string>
#include <map>
#include <algorithm>
#include "Shlwapi.h"
#include <wincrypt.h>
#include "ZLib/zlib.h"
#include "process.h"

#pragma comment (lib, "Comctl32")
#pragma comment (lib, "Shlwapi")
#pragma comment (lib, "Crypt32")

#ifdef UNICODE
#define flist std::map<std::wstring, LARGE_INTEGER>
#else
#define flist std::map<std::string, LARGE_INTEGER>
#endif
flist filelist;

LARGE_INTEGER total_size = {0, 0};
DWORD total_count = 0;
HWND hWnd = NULL;
bool stopper = false;
bool in_progress = false;
DWORD unarchsize = 0;
LPVOID unarch_ptr = NULL;
bool autounpack = true;

void EnableControls(bool en)
{
	in_progress = !en;
	EnableWindow(GetDlgItem(hWnd, IDC_PASS1), en);
	EnableWindow(GetDlgItem(hWnd, IDC_PASS2), en);
	EnableWindow(GetDlgItem(hWnd, IDC_QUERY), en);
	EnableWindow(GetDlgItem(hWnd, IDC_LEVEL), en);
	EnableWindow(GetDlgItem(hWnd, IDC_CREATE), en);
	EnableWindow(GetDlgItem(hWnd, IDC_REMOVE), en);
	EnableWindow(GetDlgItem(hWnd, IDC_NAME), en);
	EnableWindow(GetDlgItem(hWnd, IDC_CLEAR), en);
	EnableWindow(GetDlgItem(hWnd, IDC_AUTO), en);
	EnableWindow(GetDlgItem(hWnd, IDC_STOP), !en);
}

DWORD GetSaveName(HWND hWnd, HINSTANCE hInstance, TCHAR* outbuf, const TCHAR* filter, const TCHAR* title, const TCHAR* defext, DWORD* extdif)
{
	OPENFILENAME ofn;
	memset(&ofn, 0, sizeof(OPENFILENAME));

	TCHAR buf[MAX_PATH + 2];
	GetModuleFileName(hInstance, buf, 260);

	TCHAR* tmp = StrRChr(buf, NULL, TEXT('\\'));
	if(tmp != 0)
	{
		*tmp = 0;
		ofn.lpstrInitialDir = buf;
	}

	ofn.hInstance = hInstance;
	ofn.hwndOwner = hWnd;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = filter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = outbuf;
	ofn.lpstrFile[0] = 0;
	ofn.lpstrFile[1] = 0;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrTitle = title;
	ofn.lpstrDefExt = defext;
	ofn.Flags = OFN_EXPLORER | OFN_DONTADDTORECENT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_NONETWORKBUTTON | OFN_PATHMUSTEXIST | OFN_CREATEPROMPT | OFN_OVERWRITEPROMPT | OFN_NOREADONLYRETURN;

	DWORD tmp1 = GetSaveFileName(&ofn);
	if(extdif)
		*extdif = ofn.Flags & OFN_EXTENSIONDIFFERENT;

	return tmp1;
}

DWORD crc32(DWORD old_value, LPCVOID pbData, int cbData)
{
	const BYTE * pbBytes = (const BYTE*)pbData;
    DWORD crc32 = old_value ^ 0xFFFFFFFFu;
    for(int i = 0; i < cbData; i++)
	{
		BYTE b = *( pbBytes++ );
		for(int j = 0; j < 8; j++)
		{
			DWORD carry = ((crc32 & 0x80000000) ? 1 : 0) ^ (b & 0x01);
			crc32 <<= 1;
			b >>= 1;
			if(carry)
				crc32 = (crc32 ^ 0x04c11db6u) | carry;
		}
    }
    return crc32;
}

typedef struct _fileinfo
{
	LARGE_INTEGER originalSize;
	LARGE_INTEGER compressedSize;
	DWORD crc32;
	LARGE_INTEGER start;
	DWORD nlen;
}
fileinfo, *pfileinfo;

typedef struct _archive
{
	DWORD files;
	LARGE_INTEGER originalSize;
	LARGE_INTEGER compressedSize;
	WORD arch;
	DWORD dlen;
	DWORD nlen;
	BYTE no_pass;
}
archive, *parchive;

typedef struct addinfo
{
	DWORD archstart;
	DWORD filesstart;
}
addinfo, *paddinfo;

DWORD m_lMaxItemWidth = 0;

bool AddFiles(HWND hWnd, TCHAR* buf, bool add_size = true)
{
	if(filelist.find(buf) != filelist.end())
		return false;

	DWORD attr = GetFileAttributes(buf);
	if(attr & FILE_ATTRIBUTE_DIRECTORY)
	{
		WIN32_FIND_DATA ffd;

		TCHAR* dirbuf = new TCHAR[lstrlen(buf) + 3];
		lstrcpy(dirbuf, buf);
		lstrcat(dirbuf, TEXT("\\*"));

		HANDLE hFind = FindFirstFile(dirbuf, &ffd);

		if (INVALID_HANDLE_VALUE == hFind)
			return false;

		do
		{
			if(lstrcmp(ffd.cFileName, TEXT(".")) && lstrcmp(ffd.cFileName, TEXT("..")))
			{
				TCHAR* newbuf = new TCHAR[lstrlen(ffd.cFileName) + 2 + lstrlen(buf) + 1];
				lstrcpy(newbuf, buf);
				if(newbuf[lstrlen(buf)] != '\\')
					lstrcat(newbuf, TEXT("\\"));
				
				lstrcat(newbuf, ffd.cFileName);
				LARGE_INTEGER tsize;
				tsize.LowPart = ffd.nFileSizeLow;
				tsize.HighPart = ffd.nFileSizeHigh;

				if((attr & FILE_ATTRIBUTE_DIRECTORY || tsize.QuadPart > 0) && AddFiles(hWnd, newbuf, false))
				{
					filelist[newbuf].QuadPart = tsize.QuadPart;
					total_size.QuadPart += tsize.QuadPart;
				}

				delete[] newbuf;
			}
		}
		while(FindNextFile(hFind, &ffd) != 0);

		FindClose(hFind);

		delete[] dirbuf;

		return false;
	}


	if(add_size)
	{
		WIN32_FIND_DATA ffd;
		HANDLE hFind = FindFirstFile(buf, &ffd);

		if (INVALID_HANDLE_VALUE == hFind)
			return false;

		if(ffd.nFileSizeLow)
		{
			LARGE_INTEGER tsize;
			tsize.LowPart = ffd.nFileSizeLow;
			tsize.HighPart = ffd.nFileSizeHigh;
			total_size.QuadPart += tsize.QuadPart;
			filelist[buf].QuadPart = tsize.QuadPart;
		}

		FindClose(hFind);
	}

	total_count++;

	SendDlgItemMessage(hWnd, IDC_LIST2, LB_ADDSTRING, 0, (LPARAM)buf);
	return true;
}


void RefreshList(HWND hWnd)
{
	TCHAR* buf;
	HWND listwnd = GetDlgItem(hWnd, IDC_LIST2);

	HANDLE m_hItemFont = (HANDLE)SendMessage(listwnd, WM_GETFONT, 0, 0);
	HDC hdc = GetDC(listwnd);
	HGDIOBJ hOldFont = SelectObject(hdc, m_hItemFont);

	DWORD listcount = SendMessage(listwnd, LB_GETCOUNT, 0, 0);
	m_lMaxItemWidth = 0;

	for(DWORD j = 0; j < listcount; j++)
	{
		RECT rc;
		DWORD textlen = SendMessage(listwnd, LB_GETTEXTLEN, j, 0);
		buf = new TCHAR[textlen + 2];
		SendMessage(listwnd, LB_GETTEXT, j, (LPARAM)buf);
		DrawText(hdc, buf, -1, &rc, DT_SINGLELINE | DT_CALCRECT);
		delete[] buf;
		DWORD m = rc.right - rc.left;

		if(m > m_lMaxItemWidth)
			m_lMaxItemWidth = m;
	}

	SelectObject(hdc, hOldFont);
	ReleaseDC(GetDlgItem(hWnd, IDC_LIST2), hdc);
	BOOL bHasVScrBar = GetWindowLong(listwnd, GWL_STYLE) & WS_VSCROLL;
	SendMessage(listwnd, LB_SETHORIZONTALEXTENT, m_lMaxItemWidth + (bHasVScrBar ? GetSystemMetrics(SM_CXVSCROLL) : 4), 0);

	SendMessage(listwnd, WM_VSCROLL, SB_BOTTOM, 0);

	TCHAR buff[65], total_str[10];
	float tmp_total_size = (float)total_size.QuadPart;
	bool isbyte = true;

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}
	else
	{
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(isbyte)
	{
#ifdef UNICODE
	swprintf_s(buff, 64, L"%u, %u %s", total_count, total_size.LowPart, total_str);
#else
	sprintf_s(buff, 64, "%u, %u %s", total_count, total_size.LowPart, total_str);
#endif
	}
	else
	{
#ifdef UNICODE
	swprintf_s(buff, 64, L"%u, %.2f %s", total_count, tmp_total_size, total_str);
#else
	sprintf_s(buff, 64, "%u, %.2f %s", total_count, tmp_total_size, total_str);
#endif
	}

	SetDlgItemText(hWnd, IDC_LISTINFO, buff);
}

void ResetPb()
{
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, 0, 0);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);
}

TCHAR buf2[MAX_PATH + 2];

void archfunc(void*)
{
	TCHAR pass1[17], pass2[17];
	DWORD level = SendDlgItemMessage(hWnd, IDC_COMBO1, CB_GETCURSEL, 0, 0);

	GetDlgItemText(hWnd, IDC_PASS1, pass1, 17);
	GetDlgItemText(hWnd, IDC_PASS2, pass2, 17);

	if(pass1[0] == 0)
	{
		if(MessageBox(hWnd, TEXT("   ?"), TEXT(""), MB_ICONQUESTION | MB_YESNO) == IDNO)
		{
		EnableControls(true);
		return;
		}
	}
	
	if(pass1[0] != 0 && lstrcmp(pass1, pass2))
	{
		MessageBox(hWnd, TEXT("   ."), TEXT(""), MB_ICONEXCLAMATION);
		EnableControls(true);
		return;
	}

	HANDLE w = CreateFile(buf2, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);

	if(w == INVALID_HANDLE_VALUE)
	{
		MessageBox(hWnd, TEXT("   !"), TEXT("!"), MB_ICONERROR);
		EnableControls(true);
		return;
	}

	DWORD rd = 0, rd2 = 0, rd3 = 0;

	if(autounpack)
	{
		WriteFile(w, unarch_ptr, unarchsize, &rd, NULL);
		if(rd != unarchsize)
		{
			CloseHandle(w);
			MessageBox(hWnd, TEXT("   !"), TEXT("!"), MB_ICONERROR);
			EnableControls(true);
			return;
		}
	}

	HCRYPTPROV hc = NULL;
	HCRYPTHASH hash = NULL;
	HCRYPTKEY key_var = NULL;

	if(pass1[0] != 0)
	{
		DWORD keylen = lstrlen(pass1)*2;

		try
		{
			if(!CryptAcquireContext(&hc, NULL, NULL, 24, 0))
			{
				if(!CryptAcquireContext(&hc, NULL, NULL, 24, CRYPT_NEWKEYSET))
					throw(TEXT(" CryptAcquireContext"));
			}

			if(!CryptCreateHash(hc, CALG_SHA1, 0, 0, &hash))
				throw(TEXT(" CryptCreateHash"));

			if(!CryptHashData(hash, (BYTE*)pass1, keylen, 0))
				throw(TEXT(" CryptHashData"));

			if(!CryptDeriveKey(hc, CALG_AES_256, hash, 256<<16, &key_var))
				throw(TEXT(" CryptDeriveKey"));
		}
		catch(const TCHAR* err)
		{
			if(hash)
				CryptDestroyHash(hash);

			if(hc)
				CryptReleaseContext(hc, 0);

			CloseHandle(w);
			MessageBox(hWnd, err, TEXT("!"), 0);
			EnableControls(true);
			return;
		}

		CryptDestroyHash(hash);
	}


	TCHAR basedir[MAX_PATH];
	basedir[0] = 0;
	DWORD baselen = 0;
	DWORD count = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETCOUNT, 0, 0);

	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETRANGE32, 0, count);
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, 0, 0);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);

	SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXT, 0, (LPARAM)basedir);

	for(int i = count - 1; i >= 0; i--)
	{
		DWORD textlen = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXTLEN, i, 0);
		TCHAR* buf = new TCHAR[textlen + 2];
		SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXT, i, (LPARAM)buf);

		int j;
		for(j = textlen - 1; j >= 0; j--)
		{
			if(buf[j] == '\\')
			{
				buf[j + 1] = 0;
				break;
			}
		}

		DWORD lastslash = 0;
		
		for(DWORD k = 0; (int)k <= j + 1; k++)
		{
			if(buf[k] == '\\')
				lastslash = k;

			if(buf[k] != basedir[k] || buf[k] == 0)
			{
				basedir[lastslash] = 0;
				baselen = lastslash;
				break;
			}
		}

		delete[] buf;
	}

	if(baselen && basedir[baselen - 1] != '\\')
		baselen++;
	
	archive arch;
	fileinfo inf;
	arch.files = 0;
	arch.no_pass = pass1[0] == 0 ? 1 : 0;
	arch.originalSize.QuadPart= 0;
	arch.compressedSize.QuadPart = 0;
	arch.arch = (WORD)level;
	inf.start.QuadPart = 0;
	addinfo ainf;
	
	char fbuf[64016];
	char fbuf2[128000];
	char* filestructs = (char*)malloc(sizeof(fileinfo));
	DWORD maxslen = sizeof(fileinfo), curslen = 0;

	for(DWORD i = 0; i < count; i++)
	{
		DWORD textlen = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXTLEN, i, 0);
		TCHAR* buf = new TCHAR[textlen + 2];
		SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXT, i, (LPARAM)buf);

		SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, i + 1, 0);


		TCHAR* basebuf = buf + baselen;

		HANDLE h;
		if(INVALID_HANDLE_VALUE == (h = CreateFile(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0)))
		{
			delete[] buf;
			TCHAR err[MAX_PATH + 100];
#ifdef UNICODE
			swprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), buf);
#else
			sprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), buf);
#endif
			if(MessageBox(hWnd, err, TEXT(""), MB_ICONERROR | MB_YESNO) == IDYES)
			{
				CloseHandle(w);
				free(filestructs);
				DeleteFile(buf2);
				if(pass1[0] != 0)
				{
					CryptDestroyKey(key_var);
					CryptReleaseContext(hc, 0);
				}
				EnableControls(true);
				ResetPb();
				return;
			}
			else
			{
				continue;
			}
		}

		
		for(int j = textlen - 1 - baselen; j >= 0; j--)
		{
			if(basebuf[j] == ':')
				basebuf[j] = '_';
		}


		LARGE_INTEGER size, osize;
		size.LowPart = GetFileSize(h, (LPDWORD)&size.HighPart);
		osize.QuadPart = size.QuadPart;

		if(!size.QuadPart)
		{
			delete[] buf;
			CloseHandle(h);
			continue;
		}


		inf.nlen = lstrlen(basebuf);
		inf.originalSize.QuadPart = size.QuadPart;
		inf.compressedSize.QuadPart = 0;
		inf.crc32 = 0;
		
		SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, (LPARAM)(size.QuadPart/64000 ? size.QuadPart/64000 : 1));
		SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);


		while(size.QuadPart) try
		{
			DWORD to_read = size.QuadPart > 64000 ? 64000 : size.LowPart;
			DWORD dlen = 128000;
			DWORD crc32sum = 0;

			ReadFile(h, fbuf, to_read, &rd, NULL);
			if(rd != to_read)
				throw(TEXT("  ."));

			inf.crc32 = crc32(inf.crc32, fbuf, to_read);

			if(compress2((Bytef*)fbuf2, &dlen, (Bytef*)fbuf, to_read, level) != Z_OK)
				throw(TEXT("  gzip."));

#define Is2power(x) (!(x & (x - 1)))
#define ALIGN_DOWN(x, align) (x & ~(align - 1))
#define ALIGN_UP(x, align) ((x & (align - 1)) ? ALIGN_DOWN(x, align) + align : x) 

			memset(fbuf2 + dlen, 'A', 16);
			dlen = ALIGN_UP(dlen, 16);

			crc32sum = crc32(0, fbuf2, dlen);

			if(pass1[0] != 0 && !CryptEncrypt(key_var, 0, size.QuadPart > 64000 ? FALSE : TRUE, 0, (BYTE*)fbuf2, &dlen, dlen + 16))
				throw(TEXT(" ."));

			if(stopper)
				throw((const TCHAR*)NULL);

			WriteFile(w, &dlen, sizeof(DWORD), &rd, NULL);
			WriteFile(w, fbuf2, dlen, &rd2, NULL);
			WriteFile(w, &crc32sum, sizeof(DWORD), &rd3, NULL);
			if(rd != sizeof(DWORD) || rd2 != dlen || rd3 != sizeof(DWORD))
				throw(TEXT("  ."));

			inf.compressedSize.QuadPart += dlen + sizeof(DWORD)*2;

			size.QuadPart -= to_read;
			SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, (WPARAM)((osize.QuadPart - size.QuadPart)/64000), 0);
		}
		catch(const TCHAR* err)
		{
			delete[] buf;
			CloseHandle(h);
			free(filestructs);
			CloseHandle(w);
			DeleteFile(buf2);
			if(pass1[0] != 0)
			{
				CryptDestroyKey(key_var);
				CryptReleaseContext(hc, 0);
			}
			if(err)
				MessageBox(hWnd, err, TEXT("!"), MB_ICONERROR);
			EnableControls(true);
			ResetPb();
			return;
		}

		arch.compressedSize.QuadPart += inf.compressedSize.QuadPart;
		arch.originalSize.QuadPart += inf.originalSize.QuadPart;
		arch.files++;

		if(maxslen < curslen + sizeof(inf))
			filestructs = (char*)realloc(filestructs, curslen + sizeof(inf));
		memcpy(filestructs + curslen, &inf, sizeof(inf));
		curslen += sizeof(inf);

		if(maxslen < curslen + (textlen - baselen)*2)
			filestructs = (char*)realloc(filestructs, curslen + (textlen - baselen)*2);
		memcpy(filestructs + curslen, basebuf, (textlen - baselen)*2);
		curslen += (textlen - baselen)*2;
		
		inf.start.QuadPart = inf.start.QuadPart + inf.compressedSize.QuadPart;

		delete[] buf;

		CloseHandle(h);
	}


	if(arch.files) try
	{
		WriteFile(w, filestructs, curslen, &rd, NULL);
		if(rd != curslen)
			throw(TEXT("  ."));

		TCHAR description[256];
		TCHAR name[64];
		DWORD wr = GetDlgItemText(hWnd, IDC_QUERY, description, 256);
		arch.dlen = wr;
		DWORD wr2 = GetDlgItemText(hWnd, IDC_NAME, name, 64);
		if(!wr2)
		{
			lstrcpy(name, TEXT(""));
			wr2 = lstrlen(name);
		}

		arch.nlen = wr2;
		WriteFile(w, &arch, sizeof(arch), &rd, NULL);
		WriteFile(w, description, wr*2, &rd2, NULL);
		WriteFile(w, name, wr2*2, &rd3, NULL);

		if(rd != sizeof(arch) || rd2 != wr*2 || rd3 != wr2*2)
			throw(TEXT("  ."));

		ainf.archstart = wr*2 + wr2*2 + sizeof(archive) + sizeof(addinfo);
		ainf.filesstart = curslen + ainf.archstart;
		WriteFile(w, &ainf, sizeof(addinfo), &rd, NULL);
	}
	catch(const TCHAR* err)
	{
		free(filestructs);
		CloseHandle(w);
		DeleteFile(buf2);
		if(pass1[0] != 0)
		{
			CryptDestroyKey(key_var);
			CryptReleaseContext(hc, 0);
		}
		if(err)
			MessageBox(hWnd, err, TEXT("!"), MB_ICONERROR);
		EnableControls(true);
		ResetPb();
		return;
	}

	CloseHandle(w);

	if(!arch.files)
		DeleteFile(buf2);

	if(pass1[0] != 0)
	{
		CryptDestroyKey(key_var);
		CryptReleaseContext(hc, 0);
	}

	free(filestructs);

	ResetPb();

	arch.files
		? MessageBox(hWnd, TEXT("  ."), TEXT(""), MB_ICONINFORMATION)
		: MessageBox(hWnd, TEXT("       , ,     ."), TEXT(""), MB_ICONEXCLAMATION);

	EnableControls(true);
}


int DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		::hWnd = hWnd;
		{
			HRSRC res;
			if(!(res = FindResource(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_RCDATA1), RT_RCDATA)))
			{
				MessageBox(hWnd, TEXT("  !"), TEXT("!"), MB_ICONERROR);
				EndDialog(hWnd, 0);
				return 1;
			}

			HGLOBAL unarch;
			if(!(unarch = LoadResource(GetModuleHandle(NULL), res)))
			{
				MessageBox(hWnd, TEXT("  !"), TEXT("!"), MB_ICONERROR);
				EndDialog(hWnd, 0);
				return 1;
			}

			unarchsize = SizeofResource(GetModuleHandle(NULL), res);
			unarch_ptr = LockResource(unarch);
		}

		SendDlgItemMessage(hWnd, IDC_PASS1, EM_SETLIMITTEXT, 16, 0);
		SendDlgItemMessage(hWnd, IDC_PASS2, EM_SETLIMITTEXT, 16, 0);
		SendDlgItemMessage(hWnd, IDC_QUERY, EM_SETLIMITTEXT, 255, 0);
		SendDlgItemMessage(hWnd, IDC_NAME, EM_SETLIMITTEXT, 64, 0);
		
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT(" "));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("1"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("2"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("3"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("4"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("5"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("6"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("7"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("8"));
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_ADDSTRING, 0, (LPARAM)TEXT("9"));
		
		SendDlgItemMessage(hWnd, IDC_COMBO1, CB_SETCURSEL, 0, 0);

		SendDlgItemMessage(hWnd, IDC_AUTO, BM_SETCHECK, BST_CHECKED, 0);

		break;
		
	case WM_DROPFILES:
		if(!in_progress)
		{
			DWORD cnt;
			if((cnt = DragQueryFile((HDROP)wParam, 0xFFFFFFFF, NULL, 0)) > 0)
			{
				TCHAR* buf;

				for(size_t i = 0; i < cnt; i++)
				{
					size_t clen = DragQueryFile((HDROP)wParam, i, 0, 0) + 2;
					buf = new TCHAR[clen];
					DragQueryFile((HDROP)wParam, i, buf, clen);
					
					LockWindowUpdate(hWnd);
					AddFiles(hWnd, buf);
					LockWindowUpdate(NULL);

					delete[] buf;
				}

				RefreshList(hWnd);
			}


			DragFinish((HDROP)wParam);
		}

		break;

	case WM_COMMAND:
		switch(wParam)
		{
		case IDC_AUTO:
			autounpack = SendDlgItemMessage(hWnd, IDC_AUTO, BM_GETCHECK, 0, 0) == BST_CHECKED;
			break;
		case IDC_REMOVE:
			{
				DWORD count = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETSELCOUNT, 0, 0);
				DWORD* indexes = new DWORD[count];
				count = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETSELITEMS, count, (LPARAM)indexes);

				LockWindowUpdate(hWnd);

				for(int i = count - 1; i >= 0; i--)
				{
					DWORD textlen = SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXTLEN, indexes[i], 0);
					TCHAR* buf = new TCHAR[textlen + 2];
					SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETTEXT, indexes[i], (LPARAM)buf);
					total_size.QuadPart -= filelist[buf].QuadPart;
					filelist.erase(buf);
					delete[] buf;

					total_count--;
					
					SendDlgItemMessage(hWnd, IDC_LIST2, LB_DELETESTRING, (WPARAM)indexes[i], 0);
				}

				LockWindowUpdate(NULL);
				
				delete[] indexes;
				
				RefreshList(hWnd);
			}
			break;

		case IDC_CLEAR:
			SendDlgItemMessage(hWnd, IDC_LIST2, LB_RESETCONTENT, 0, 0);
			total_count = 0;
			total_size.QuadPart = 0;
			filelist.clear();
			RefreshList(hWnd);
			break;


		case IDC_STOP:
			if(MessageBox(hWnd, TEXT(" ?"), TEXT(""), MB_ICONQUESTION | MB_YESNO) == IDYES)
				stopper = true;
			break;


		case IDC_CREATE:
			DWORD extdif = 0;

			if(0 == SendDlgItemMessage(hWnd, IDC_LIST2, LB_GETCOUNT, 0, 0))
			{
				MessageBox(hWnd, TEXT("   ."),TEXT("!"), 0);
				return 0;
			}

#ifdef _DEBUG
			if(1 || 0 != GetSaveName(hWnd, GetModuleHandle(NULL), buf2, TEXT("Archive files\0*.exe\0All Files\0*.*\0\0"), TEXT("Save archive as..."), TEXT("exe"), &extdif))
			{
				lstrcpy(buf2, L"D:\\Users\\Documents\\Visual Studio 2008\\Projects\\archiver\\111.exe");
#else
			if(0 != GetSaveName(hWnd, GetModuleHandle(NULL), buf2, autounpack ? TEXT("  (*.exe)\0*.exe\0  (*.*)\0*.*\0\0") : TEXT("  (*.dxa)\0*.dxa\0All Files (*.*)\0*.*\0\0"), TEXT("  ..."), autounpack ? TEXT("exe") : TEXT("dxa"), &extdif))
			{
#endif
				if(extdif)
				{
					if(lstrlen(buf2) < MAX_PATH - 4)
						lstrcat(buf2, autounpack ? TEXT(".exe") : TEXT(".dxa"));
				}

				EnableControls(false);
				stopper = false;
				HANDLE archproc = (HANDLE)_beginthread(archfunc, 0, 0);
				if((int)archproc == -1)
				{
					MessageBox(hWnd, TEXT("   ."), TEXT("!"), 0);
					return 0;
				}
			}
			break;
		}
		break;

	case WM_CLOSE:
		if(in_progress)
			MessageBox(hWnd, TEXT("  "), TEXT(""), MB_ICONEXCLAMATION);
		else
			EndDialog(hWnd,0);
		break;
	}

	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	INITCOMMONCONTROLSEX ctrl;

	ctrl.dwSize = sizeof(INITCOMMONCONTROLSEX);
	ctrl.dwICC = ICC_DATE_CLASSES | ICC_INTERNET_CLASSES | ICC_PAGESCROLLER_CLASS | ICC_COOL_CLASSES;

	InitCommonControlsEx(&ctrl);
	
	DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(Arch), 0, (DLGPROC) DlgProc, 0);

	return 0;
}