#include "windows.h"
#include "stdio.h"
#include "resource.h"
#include "Commctrl.h"
#include "Shlwapi.h"
#include <string>
#include <map>
#include <algorithm>
#include <wincrypt.h>
#include "../archiver/ZLib/zlib.h"
#include "process.h"
#include "Shtypes.h"
#include "Shlobj.h"
#include "Strsafe.h"

#pragma comment (lib, "Comctl32")
#pragma comment (lib, "Crypt32")
#pragma comment (lib, "Shlwapi")
#pragma comment (lib, "Shell32")

HWND hWnd = NULL;
bool stopper = false;
bool in_progress = false;
HANDLE w;
TCHAR unarch_path[MAX_PATH + 2];

typedef struct _fileinfo
{
	LARGE_INTEGER originalSize;
	LARGE_INTEGER compressedSize;
	DWORD crc32;
	LARGE_INTEGER start;
	DWORD nlen;
}
fileinfo, *pfileinfo;

typedef struct _archive
{
	DWORD files;
	LARGE_INTEGER originalSize;
	LARGE_INTEGER compressedSize;
	WORD arch;
	DWORD dlen;
	DWORD nlen;
	BYTE no_pass;
}
archive, *parchive;



DWORD GetOpenName(HWND hWnd, HINSTANCE hInstance, TCHAR* outbuf, const TCHAR* filter, const TCHAR* title)
{
	OPENFILENAME ofn;
	memset(&ofn, 0, sizeof(OPENFILENAME));

	TCHAR buf[MAX_PATH + 2];
	GetModuleFileName(hInstance, buf, 260);

	TCHAR* tmp = StrRChr(buf, NULL, L'\\');
	if(tmp != 0)
	{
		*tmp = 0;
		ofn.lpstrInitialDir = buf;
	}

	ofn.hInstance = hInstance;
	ofn.hwndOwner = hWnd;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = filter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = outbuf;
	ofn.lpstrFile[0] = 0;
	ofn.lpstrFile[1] = 0;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrTitle = title;
	ofn.Flags = OFN_EXPLORER | OFN_DONTADDTORECENT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_NONETWORKBUTTON | OFN_PATHMUSTEXIST;

	return GetOpenFileName(&ofn);
}

DWORD crc32(DWORD old_value, LPCVOID pbData, int cbData)
{
	const BYTE * pbBytes = (const BYTE*)pbData;
    DWORD crc32 = old_value ^ 0xFFFFFFFFu;
    for(int i = 0; i < cbData; i++)
	{
		BYTE b = *( pbBytes++ );
		for(int j = 0; j < 8; j++)
		{
			DWORD carry = ((crc32 & 0x80000000) ? 1 : 0) ^ (b & 0x01);
			crc32 <<= 1;
			b >>= 1;
			if(carry)
				crc32 = (crc32 ^ 0x04c11db6u) | carry;
		}
    }
    return crc32;
}


#ifdef UNICODE
#define flist std::map<DWORD, std::pair<std::wstring, fileinfo>>
#else
#define flist std::map<DWORD, std::pair<std::string, fileinfo>>
#endif
flist filelist;

typedef struct addinfo
{
	DWORD archstart;
	DWORD filesstart;
}
addinfo, *paddinfo;

addinfo ainf;
archive arch;




void get_sizes(LARGE_INTEGER originalSize, LARGE_INTEGER compressedSize, TCHAR* buf, TCHAR* name)
{
	float tmp_total_size = (float)originalSize.QuadPart;
	bool isbyte = true;
	TCHAR total_str[10];
	TCHAR buf2[36];
	
	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(isbyte)
	{
#ifdef UNICODE
		swprintf_s(buf2, 35, TEXT(", : %u "), originalSize.LowPart);
#else
		sprintf_s(buf2, 35, TEXT(", : %u "), originalSize.LowPart);
#endif
	}
	else
	{
#ifdef UNICODE
		swprintf_s(buf2, 35, TEXT(", : %.2f %s"), tmp_total_size, total_str);
#else
		sprintf_s(buf2, 35, TEXT(", : %.2f %s"), tmp_total_size, total_str);
#endif
	}



	tmp_total_size = (float)compressedSize.QuadPart;
	isbyte = true;

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}


	if(isbyte)
	{
#ifdef UNICODE
		swprintf_s(buf, 127, TEXT("%s;   : %u %s"), name, compressedSize.LowPart, buf2);
#else
		sprintf_s(buf, 127, TEXT("%s;   : %u %s"), name, compressedSize.LowPart, buf2);
#endif
	}
	else
	{
#ifdef UNICODE
		swprintf_s(buf, 127, TEXT("%s;   : %.2f %s%s"), name, tmp_total_size, total_str, buf2);
#else
		sprintf_s(buf, 127, TEXT("%s;   : %.2f %s%s"), name, tmp_total_size, total_str, buf2);
#endif
	}
}


void EnableControls(bool en)
{
	in_progress = !en;
	EnableWindow(GetDlgItem(hWnd, IDC_PASS1), en);
	EnableWindow(GetDlgItem(hWnd, IDC_CHOOSEPATH), en);
	EnableWindow(GetDlgItem(hWnd, IDC_FILELIST), en);
	EnableWindow(GetDlgItem(hWnd, IDC_UNPACK), en);
	EnableWindow(GetDlgItem(hWnd, IDC_CANCEL), !en);
	EnableWindow(GetDlgItem(hWnd, IDC_SELECTALL), en);
}

void ResetPb()
{
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, 0, 0);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);
}

void archfunc(void*)
{
	TCHAR pass1[17];
	GetDlgItemText(hWnd, IDC_PASS1, pass1, 17);
	if(arch.no_pass == 0 && pass1[0] == 0)
	{
		MessageBox(hWnd, TEXT(" ."), TEXT(""), MB_ICONEXCLAMATION);
		EnableControls(true);
		return;
	}

	HCRYPTPROV hc = NULL;
	HCRYPTHASH hash = NULL;
	HCRYPTKEY key_var = NULL;

	if(arch.no_pass == 0)
	{
		DWORD keylen = lstrlen(pass1)*2;

		try
		{
			if(!CryptAcquireContext(&hc, NULL, NULL, 24, 0))
			{
				if(!CryptAcquireContext(&hc, NULL, NULL, 24, CRYPT_NEWKEYSET))
					throw(TEXT(" CryptAcquireContext"));
			}

			if(!CryptCreateHash(hc, CALG_SHA1, 0, 0, &hash))
				throw(TEXT(" CryptCreateHash"));

			if(!CryptHashData(hash, (BYTE*)pass1, keylen, 0))
				throw(TEXT(" CryptHashData"));

			if(!CryptDeriveKey(hc, CALG_AES_256, hash, 256<<16, &key_var))
				throw(TEXT(" CryptDeriveKey"));
		}
		catch(const TCHAR* err)
		{
			if(hash)
				CryptDestroyHash(hash);

			if(hc)
				CryptReleaseContext(hc, 0);

			MessageBox(hWnd, err, TEXT("!"), 0);
			EnableControls(true);
			return;
		}

		CryptDestroyHash(hash);
	}

	DWORD count = SendDlgItemMessage(hWnd, IDC_FILELIST, LB_GETSELCOUNT, 0, 0);
	DWORD* indexes = new DWORD[count];
	count = SendDlgItemMessage(hWnd, IDC_FILELIST, LB_GETSELITEMS, count, (LPARAM)indexes);

	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETRANGE32, 0, count);
	SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, 0, 0);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, 1);
	SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);
	
	char fbuf[128000];
	char fbuf2[128000];
	
	DWORD rd = 0;
	
	for(DWORD i = 0; i < count; i++)
	{
		SendDlgItemMessage(hWnd, IDC_PROGRESS1, PBM_SETPOS, i + 1, 0);

		TCHAR basebuf[MAX_PATH + 2];
		lstrcpy(basebuf, filelist[indexes[i]].first.c_str());
		DWORD textlen = lstrlen(basebuf);
		TCHAR fullpath[MAX_PATH + 2];

		for(DWORD j = 0; j < textlen; j++)
		{
			if(basebuf[j] == '\\')
			{
				basebuf[j] = 0;
				lstrcpy(fullpath, unarch_path);
				StringCchCat(fullpath, MAX_PATH, basebuf);
				DWORD attr = GetFileAttributes(fullpath);

				if((attr == INVALID_FILE_ATTRIBUTES && !CreateDirectory(fullpath, NULL)) || !(attr & FILE_ATTRIBUTE_DIRECTORY))
				{
					TCHAR err[MAX_PATH + 100];
#ifdef UNICODE
					swprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), fullpath);
#else
					sprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), fullpath);
#endif
					if(MessageBox(hWnd, err, TEXT(""), MB_ICONERROR | MB_YESNO) == IDYES)
					{
						if(arch.no_pass == 0)
						{
							CryptDestroyKey(key_var);
							CryptReleaseContext(hc, 0);
						}

						EnableControls(true);
						ResetPb();
						return;
					}
					else
					{
						continue;
					}
				}

				basebuf[j] = '\\';
			}
		}
		
		lstrcpy(fullpath, unarch_path);
		StringCchCat(fullpath, MAX_PATH, basebuf);

		if(GetFileAttributes(fullpath) != INVALID_FILE_ATTRIBUTES)
		{
			TCHAR err[MAX_PATH + 100];
#ifdef UNICODE
			swprintf_s(err, MAX_PATH + 99, TEXT(" %s  .\r\n?"), fullpath);
#else
			sprintf_s(err, MAX_PATH + 99, TEXT(" %s  .\r\n?"), fullpath);
#endif
			if(MessageBox(hWnd, err, TEXT(""), MB_ICONQUESTION | MB_YESNO) == IDNO)
				continue;
		}
		
		HANDLE h;
		if(INVALID_HANDLE_VALUE == (h = CreateFile(fullpath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0)))
		{
			TCHAR err[MAX_PATH + 100];
#ifdef UNICODE
			swprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), fullpath);
#else
			sprintf_s(err, MAX_PATH + 99, TEXT("    %s.\r\n ?"), fullpath);
#endif
			if(MessageBox(hWnd, err, TEXT(""), MB_ICONERROR | MB_YESNO) == IDYES)
			{
				if(arch.no_pass == 0)
				{
					CryptDestroyKey(key_var);
					CryptReleaseContext(hc, 0);
				}

				EnableControls(true);
				ResetPb();
				return;
			}
			else
			{
				continue;
			}
		}


		fileinfo finfo = filelist[indexes[i]].second;
		LARGE_INTEGER size, osize;
		size.QuadPart = finfo.compressedSize.QuadPart;
		osize.QuadPart = size.QuadPart;

		if(!size.QuadPart)
		{
			CloseHandle(h);
			continue;
		}
		
		SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETRANGE32, 0, (LPARAM)(size.QuadPart/64000 ? size.QuadPart/64000 : 1));
		SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);

		DWORD totalcrc32 = 0;
		
		LARGE_INTEGER sm = {0, 0}, fsize = {0, 0};
		for(DWORD k = 0; k < indexes[i]; k++)
			sm.QuadPart += filelist[k].second.compressedSize.QuadPart;
		
		fsize.LowPart = GetFileSize(w, (LPDWORD)&fsize.HighPart);

		fsize.QuadPart = fsize.QuadPart - arch.compressedSize.QuadPart - ainf.filesstart + sm.QuadPart;

		SetFilePointer(w, fsize.LowPart, &fsize.HighPart, FILE_BEGIN);

		LARGE_INTEGER read_full = {0, 0};

		while(size.QuadPart) try
		{
			DWORD dlen = 128000;
			DWORD crc32sum = 0;

			DWORD to_read;
			ReadFile(w, &to_read, sizeof(DWORD), &rd, NULL);
			if(rd != sizeof(DWORD))
				throw(TEXT("  ."));

			DWORD orig = to_read;

			ReadFile(w, fbuf, to_read, &rd, NULL);
			if(rd != to_read)
				throw(TEXT("  ."));
			
			if(arch.no_pass == 0 && !CryptDecrypt(key_var, 0, read_full.QuadPart + (__int64)to_read + (__int64)sizeof(DWORD)*2 < finfo.compressedSize.QuadPart ? FALSE : TRUE, 0, (BYTE*)fbuf, &to_read))
				throw(TEXT(" .  ,   ."));

			crc32sum = crc32(0, fbuf, to_read);

			DWORD tempcrc = 0;

			ReadFile(w, &tempcrc, sizeof(DWORD), &rd, NULL);
			if(rd != sizeof(DWORD))
				throw(TEXT("  ."));
			
			if(crc32sum != tempcrc)
				throw(TEXT("   (   )."));

			if(uncompress((Bytef*)fbuf2, &dlen, (Bytef*)fbuf, to_read) != Z_OK)
				throw(TEXT("  gzip."));

			totalcrc32 = crc32(totalcrc32, fbuf2, dlen);

			if(stopper)
				throw((const TCHAR*)NULL);

			WriteFile(h, fbuf2, dlen, &rd, NULL);
			if(rd != dlen)
				throw(TEXT("   ."));
			
			size.QuadPart -= (orig + sizeof(DWORD)*2);
			read_full.QuadPart += orig + sizeof(DWORD)*2;
			SendDlgItemMessage(hWnd, IDC_PROGRESS2, PBM_SETPOS, (WPARAM)((osize.QuadPart - size.QuadPart)/64000), 0);
		}
		catch(const TCHAR* err)
		{
			CloseHandle(h);
			DeleteFile(fullpath);

			if(arch.no_pass == 0)
			{
				CryptDestroyKey(key_var);
				CryptReleaseContext(hc, 0);
			}

			if(err)
				MessageBox(hWnd, err, TEXT("!"), MB_ICONERROR);
			EnableControls(true);
			ResetPb();
			return;
		}

		CloseHandle(h);

		if(totalcrc32 != finfo.crc32)
		{
			DeleteFile(fullpath);

			if(arch.no_pass == 0)
			{
				CryptDestroyKey(key_var);
				CryptReleaseContext(hc, 0);
			}

			MessageBox(hWnd, TEXT(" ,    ."), TEXT("!"), MB_ICONERROR);
			EnableControls(true);
			ResetPb();
			return;
		}
	}


	if(arch.no_pass == 0)
	{
		CryptDestroyKey(key_var);
		CryptReleaseContext(hc, 0);
	}


	ResetPb();
	
	MessageBox(hWnd, TEXT("   ."), TEXT(""), MB_ICONINFORMATION);

	EnableControls(true);
}


void RefreshList(HWND hWnd)
{
	TCHAR* buf;
	HWND listwnd = GetDlgItem(hWnd, IDC_FILELIST);

	HANDLE m_hItemFont = (HANDLE)SendMessage(listwnd, WM_GETFONT, 0, 0);
	HDC hdc = GetDC(listwnd);
	HGDIOBJ hOldFont = SelectObject(hdc, m_hItemFont);

	DWORD listcount = SendMessage(listwnd, LB_GETCOUNT, 0, 0);
	DWORD m_lMaxItemWidth = 0;

	for(DWORD j = 0; j < listcount; j++)
	{
		RECT rc;
		DWORD textlen = SendMessage(listwnd, LB_GETTEXTLEN, j, 0);
		buf = new TCHAR[textlen + 2];
		SendMessage(listwnd, LB_GETTEXT, j, (LPARAM)buf);
		DrawText(hdc, buf, -1, &rc, DT_SINGLELINE | DT_CALCRECT);
		delete[] buf;
		DWORD m = rc.right - rc.left;

		if(m > m_lMaxItemWidth)
			m_lMaxItemWidth = m;
	}

	SelectObject(hdc, hOldFont);
	ReleaseDC(GetDlgItem(hWnd, IDC_FILELIST), hdc);
	BOOL bHasVScrBar = GetWindowLong(listwnd, GWL_STYLE) & WS_VSCROLL;
	SendMessage(listwnd, LB_SETHORIZONTALEXTENT, m_lMaxItemWidth + (bHasVScrBar ? GetSystemMetrics(SM_CXVSCROLL) : 4), 0);

	SendMessage(listwnd, WM_VSCROLL, SB_BOTTOM, 0);
}


void PreLoad(const TCHAR* fname)
{
	w = CreateFile(fname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0);
	if(w == INVALID_HANDLE_VALUE)
	{
		MessageBox(0, TEXT("   ."), TEXT(""), MB_ICONERROR);
		EndDialog(hWnd, 0);
		return;
	}

	TCHAR* tmp = StrRChr(fname, NULL, TEXT('\\'));
	if(tmp != 0)
	{
		*(tmp+1) = 0;
		SetDlgItemText(hWnd, IDC_UNPACKPATH, fname);
	}

	DWORD rv = 0;
	LARGE_INTEGER size, tmpsize;
	size.LowPart = GetFileSize(w, (LPDWORD)&size.HighPart);
	tmpsize.QuadPart = size.QuadPart;

	tmpsize.QuadPart = size.QuadPart - sizeof(addinfo);

	SetFilePointer(w, tmpsize.LowPart, &tmpsize.HighPart, FILE_BEGIN);

	ReadFile(w, &ainf, sizeof(addinfo), &rv, NULL);
	if(rv != sizeof(addinfo) || ainf.archstart > size.QuadPart || ainf.filesstart > size.QuadPart)
		throw(TEXT("   ."));

	tmpsize.QuadPart = size.QuadPart - ainf.archstart;
	SetFilePointer(w, tmpsize.LowPart, &tmpsize.HighPart, FILE_BEGIN);
	ReadFile(w, &arch, sizeof(archive), &rv, NULL);
	if(rv != sizeof(archive) || arch.dlen > 255 || arch.dlen > ainf.archstart || arch.arch > 9 || arch.nlen > 64 || arch.nlen == 0 || arch.compressedSize.QuadPart == 0 || arch.originalSize.QuadPart == 0)
		throw(TEXT("   ."));

	TCHAR* buf = new TCHAR[arch.dlen + 1];
	buf[arch.dlen] = 0;
	ReadFile(w, buf, arch.dlen*2, &rv, NULL);
	if(rv != arch.dlen*2)
		throw(TEXT("   ."));

	SetDlgItemText(hWnd, IDC_INFO, buf);

	delete[] buf;


	buf = new TCHAR[arch.nlen + 1];
	buf[arch.nlen] = 0;
	ReadFile(w, buf, arch.nlen*2, &rv, NULL);
	if(rv != arch.nlen*2)
		throw(TEXT("   ."));

	SetWindowText(hWnd, buf);

	delete[] buf;

	buf = new TCHAR[128];
	TCHAR total_str[10];
	TCHAR buf2[36];


	float tmp_total_size = (float)arch.originalSize.QuadPart;
	bool isbyte = true;

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(isbyte)
	{
#ifdef UNICODE
		swprintf_s(buf2, 35, TEXT(", : %u "), arch.originalSize.LowPart);
#else
		sprintf_s(buf2, 35, TEXT(", : %u "), arch.originalSize.LowPart);
#endif
	}
	else
	{
#ifdef UNICODE
		swprintf_s(buf2, 35, TEXT(", : %.2f %s"), tmp_total_size, total_str);
#else
		sprintf_s(buf2, 35, TEXT(", : %.2f %s"), tmp_total_size, total_str);
#endif
	}

	tmp_total_size = (float)arch.compressedSize.QuadPart;
	isbyte = true;

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(tmp_total_size > 1024)
	{
		isbyte = false;
		tmp_total_size /= 1024;
		lstrcpy(total_str, TEXT(""));
	}

	if(isbyte)
	{
#ifdef UNICODE
		swprintf_s(buf, 127, TEXT("  : %u,   : %u %s"), arch.files, arch.compressedSize.LowPart, buf2);
#else
		sprintf_s(buf, 127, TEXT("  : %u,   : %u %s"), arch.files, arch.compressedSize.LowPart, buf2);
#endif
	}
	else
	{
#ifdef UNICODE
		swprintf_s(buf, 127, TEXT("  : %u,   : %.2f %s%s"), arch.files, tmp_total_size, total_str, buf2);
#else
		sprintf_s(buf, 127, TEXT("  : %u,   : %.2f %s%s"), arch.files, tmp_total_size, total_str, buf2);
#endif
	}

	SetDlgItemText(hWnd, IDC_ARCHINFO, buf);

#ifdef UNICODE
	swprintf_s(buf, 127, TEXT(" : %u"), arch.arch);
#else
	sprintf_s(buf, 127, TEXT(" : %u"), arch.arch);
#endif
	SetDlgItemText(hWnd, IDC_LEVEL, buf);

	delete[] buf;

	fileinfo finf;

	tmpsize.QuadPart = size.QuadPart - ainf.filesstart;
	SetFilePointer(w, tmpsize.LowPart, &tmpsize.HighPart, FILE_BEGIN);

	LARGE_INTEGER check_size = {0, 0}, check_size2 = {0, 0};

	for(DWORD i = 0; i < arch.files; i++)
	{
		ReadFile(w, &finf, sizeof(fileinfo), &rv, NULL);
		if(rv != sizeof(fileinfo) || finf.nlen > MAX_PATH || !finf.compressedSize.QuadPart || !finf.originalSize.QuadPart)
			throw(TEXT("   ."));

		buf = new TCHAR[finf.nlen + 1];
		buf[finf.nlen] = 0;
		ReadFile(w, buf, finf.nlen*2, &rv, NULL);
		if(rv != finf.nlen*2)
		{
			delete[] buf;
			throw(TEXT("   ."));
		}


		filelist[i] = std::make_pair(buf, finf);
		check_size.QuadPart += finf.compressedSize.QuadPart;
		check_size2.QuadPart += finf.originalSize.QuadPart;

		TCHAR* buf2 = new TCHAR[128 + MAX_PATH];

		get_sizes(finf.originalSize, finf.compressedSize, buf2, buf);
		SendDlgItemMessage(hWnd, IDC_FILELIST, LB_ADDSTRING, 0, (LPARAM)buf2);

		delete[] buf;
		delete[] buf2;
	}

	if(check_size.QuadPart != arch.compressedSize.QuadPart || check_size2.QuadPart != arch.originalSize.QuadPart)
		throw(TEXT("   ."));

	if(arch.no_pass == 1)
		EnableWindow(GetDlgItem(hWnd, IDC_PASS1), FALSE);
}

int DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		::hWnd = hWnd;
		TCHAR fname[MAX_PATH + 1];

		GetModuleFileName(NULL, fname, MAX_PATH);

		try
		{
			PreLoad(fname);
		}
		catch(const TCHAR*)
		{
			SendDlgItemMessage(hWnd, IDC_FILELIST, LB_RESETCONTENT, 0, 0);
			filelist.clear();
			CloseHandle(w);

			try
			{
				if(!GetOpenName(hWnd, GetModuleHandle(NULL), fname, TEXT("  (*.dxa)\0*.dxa\0  (*.exe)\0*.exe\0  (*.*)\0*.*\0\0"), TEXT(" ...")))
					throw((const TCHAR*)NULL);

				PreLoad(fname);
			}
			catch(const TCHAR* err)
			{
				if(err)
					MessageBox(0, err, TEXT(""), MB_ICONERROR);
				EndDialog(hWnd, 0);
				return 1;
			}
		}

		SendDlgItemMessage(hWnd, IDC_PASS1, EM_SETLIMITTEXT, 16, 0);
		RefreshList(hWnd);
		break;

	case WM_COMMAND:
		switch(wParam)
		{
		case IDC_CANCEL:
			if(MessageBox(hWnd, TEXT("  ?"), TEXT(""), MB_ICONQUESTION | MB_YESNO) == IDYES)
				stopper = true;
			break;

		case IDC_SELECTALL:
			{
				DWORD end = SendDlgItemMessage(hWnd, IDC_FILELIST, LB_GETCOUNT, 0, 0);
				if(end)
					SendDlgItemMessage(hWnd, IDC_FILELIST, LB_SELITEMRANGEEX, 0, end);
			}
			break;
		case IDC_CHOOSEPATH:
			CoInitialize(NULL);
			{
				BROWSEINFO folder;
				TCHAR buf[MAX_PATH + 2];
				folder.hwndOwner = hWnd;
				folder.lpszTitle = TEXT("   ");
				folder.pidlRoot = NULL;
				folder.pszDisplayName = buf;
				folder.ulFlags = BIF_DONTGOBELOWDOMAIN | BIF_NEWDIALOGSTYLE | BIF_RETURNONLYFSDIRS;
				folder.lpfn = NULL;
				folder.iImage = 0;
				folder.lParam = 0;
				LPITEMIDLIST list;
				if((list = SHBrowseForFolder(&folder)) != NULL)
				{
					SHGetPathFromIDList(list, buf);
					if(buf[lstrlen(buf) - 1] != TEXT('\\'))
						lstrcat(buf, TEXT("\\"));

					SetDlgItemText(hWnd, IDC_UNPACKPATH, buf);
					CoTaskMemFree(list);
				}
			}
			CoUninitialize();
			break;

		case IDC_UNPACK:
			if(0 == SendDlgItemMessage(hWnd, IDC_FILELIST, LB_GETSELCOUNT, 0, 0))
			{
				MessageBox(hWnd, TEXT("   ."),TEXT("!"), 0);
				return 0;
			}

			GetDlgItemText(hWnd, IDC_UNPACKPATH, unarch_path, MAX_PATH + 2);
			if(!(GetFileAttributes(unarch_path) & FILE_ATTRIBUTE_DIRECTORY))
			{
				MessageBox(hWnd, TEXT("    ."),TEXT("!"), 0);
				return 0;
			}


			EnableControls(false);
			stopper = false;
			HANDLE archproc = (HANDLE)_beginthread(archfunc, 0, 0);
			if((int)archproc == -1)
			{
				MessageBox(hWnd, TEXT("   ."), TEXT("!"), 0);
				return 0;
			}
			break;
		}
		break;

	case WM_CLOSE:
		if(in_progress)
		{
			MessageBox(hWnd, TEXT("  ."), TEXT(""), MB_ICONEXCLAMATION);
		}
		else
		{
			CloseHandle(w);
			EndDialog(hWnd,0);
		}
		break;
	}
	
	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	INITCOMMONCONTROLSEX ctrl;

	ctrl.dwSize = sizeof(INITCOMMONCONTROLSEX);
	ctrl.dwICC = ICC_DATE_CLASSES | ICC_INTERNET_CLASSES | ICC_PAGESCROLLER_CLASS | ICC_COOL_CLASSES;

	InitCommonControlsEx(&ctrl);
	
	DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(UnArch), 0, (DLGPROC) DlgProc, 0);

	return 0;
}